% ===================================================================================================================================
% Copyright notice:
% These MATLAB-routines have been developed by Guido Terra in answer to a question by Robert Nottrot raised through "www.wisfaq.nl".
% You are free to use, reproduce and distribute them for non-commercial use as long as this copyright notice is included verbatim.
% If you want to modify the code, send a copy of your changes to the author at "terra@nioz.nl".
% Moreover, you are strongly encouraged to contact me to report any bugs, mistakes and other errors.
% The routines have been written for MATLAB-release 13; at least one error is known to occur when using release 12.
% A complete set of files would contain the following 13 files:
%   aantalpatronen.m, bepaalcykels.m, cykellengtes.m,  bepaalpatronen.m, bepaalallebanen.m, bepaalbanen.m
%   equivpatronen.m, diagequivpatronen.m, tekenpatroon.m, patroongetal.m, schrijfpatronen.m
%   patronendemo.m, patronendemo.fig
% This ends the copyright notice.
% ===================================================================================================================================

% Deze matlab-functie bepaalt de lengtes van de cykels van een permutatie.
% De permutatie wordt gegeven door de vector p, waarin p(i) het beeld van i is.
% 
% De uitvoer is een vector met de lengtes van de cykels erin.

function len = cykellengtes(p)

% checken dat p een permutatie is.
p = reshape(p, 1, prod(size(p)));
if ~isequal(sort(p), [1:length(p)])
    error(['de invoer p = [' num2str(p) '] is geen permutatie!'])
end

% cykels bepalen
decykels = bepaalcykels(p);

% lengtes aflezen
len = zeros(1,length(decykels));
for j=1:length(decykels)
    len(j) = length(decykels{j});
end