% ===================================================================================================================================
% Copyright notice:
% These MATLAB-routines have been developed by Guido Terra in answer to a question by Robert Nottrot raised through "www.wisfaq.nl".
% You are free to use, reproduce and distribute them for non-commercial use as long as this copyright notice is included verbatim.
% If you want to modify the code, send a copy of your changes to the author at "terra@nioz.nl".
% Moreover, you are strongly encouraged to contact me to report any bugs, mistakes and other errors.
% The routines have been written for MATLAB-release 13; at least one error is known to occur when using release 12.
% A complete set of files would contain the following 13 files:
%   aantalpatronen.m, bepaalcykels.m, cykellengtes.m,  bepaalpatronen.m, bepaalallebanen.m, bepaalbanen.m
%   equivpatronen.m, diagequivpatronen.m, tekenpatroon.m, patroongetal.m, schrijfpatronen.m
%   patronendemo.m, patronendemo.fig
% This ends the copyright notice.
% ===================================================================================================================================

% Deze matlab-functie geeft het getal dat het ingevoerde patroon van tekens op m bij n vakjes representeerd.
% In die zin is het de inverse van de routine tekenpatroon, met dien verstande dat
% patroongetal(tekenpatroon(q, m,n, [1,1], vulling),vulling) = q.
% INVOER:   - char-array s met slechts de tekens vulling(1) en vulling(2).
%               De eerste twee dimensies worden als m en n geinterpreteerd, de rest als verschillende patronen.
%           - als er geen vulling wordt meegegeven dan wordt gezocht op '\/', '-|', 'x+', 'sz', '<> of 'MW', of elk ander tweetal (op ascii-volgorde).
%               vulling = 'bin' doet zoeken naar '01' voor compatibiliteit met tekenpatroon.
% UITVOER:  - de getallen q waarvan de binaire ontwikkelingen de patronen p representeren.
%           - de afmetingen m,n van het patroon, zodat die eventueel in tekenpatroon kan worden ingevoerd.

function [q,m,n] = tekenpatroon(p, vulling)

% checken dat p een patroon is:
altvulling = unique(p); altvulling = reshape(altvulling, 1, prod(size(altvulling)));
if ~ischar(p) | length(altvulling) > 2 error('Input p is not a pattern of at most two characters'); end
% DEFAULT-waarde voor vulling
if nargin < 2
    switch altvulling
    case {'/\','/','\','\/'}
        vulling = '\/';
    case {'-|','-','|','|-'}
        vulling = '-|';
    case {'+x','x','+','x+'}
        vulling = 'x+';
    case {'sz','s','z','zs'}
        vulling = 'sz';
    case {'<>','<','>','><'}
        vulling = '<>';
    case {'MW','M','W','WM'}
        vulling = 'MW';
    case {'01','0','1','10'}
        vulling = '01';
    otherwise
        if length(altvulling)==1 vulling = [altvulling ' ']; else vulling = altvulling; end
end;end
if isequal(vulling, 'bin') vulling = '01'; end
% checken of vulling de twee tekens is waaruit het patroon is opgebouwd.
if length(vulling) < 2 error('You must specify both characters of which the pattern is composed!'); end
% we moeten de JIF-acceleratie even uitschakelen om een bug in ismember te ondervangen.
feature accel off;
if ismember(p,vulling(1:2))
    % als alle elementen van p in vulling(1:2) zitten dan is het goed
else
    error(['Input pattern p is not composed of the characters vulling(1) = ' vulling(1) ' and vulling(2) = ' vulling(2) '!'])
end
feature accel on;


% Het patroon omzetten in enen en nullen.
tp = p==vulling(1); % mocht je enen en nullen moeten omdraaien, dan moet je even onthouden waar ze staan.
p(p==vulling(2))='1';
p(tp) = '0';

% Het ontstane binaire getal uitrekenen.
afmetingen = size(p); m = afmetingen(1); n = afmetingen(2);
afmetingen(1:2)= []; if length(afmetingen)<2 afmetingen = [afmetingen ones(1,2-length(afmetingen))]; end
q = reshape(bin2dec(reshape(p,m*n,prod(afmetingen))'),afmetingen);